<?php
/*
* Plugin Name: Advanced Chart for Elementor
* Plugin URI: https://kapasias.com/advanced-charts-for-elementor/
* Description:  Advanced Chart for Elementor and proudly developed by KAP ASIAs Team.
* Version: 1.0.0
* Author: KAP ASIAs
* Author URI: http://kapasias.com
* Text Domain: acfe
* Elementor tested up to: 2.9.8
*/

// Prevent direct access to files
if (!defined('ABSPATH')) {
    exit;
}
// Plugin version
defined( 'ACFE_VERSION' ) or define( 'ACFE_VERSION', '1.0.0' );
define('ACFE_PATH', plugin_dir_path(__FILE__));
define('ACFE_URL', plugin_dir_url(__FILE__));

// Check elementor
require_once ACFE_PATH . 'include/acfe-plugin-check.php';

class Ka_Advanced_Chart_Addon {

    private static $_instance = null;

    public static function get_instance() {
        if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
    }

    public function __construct() {
		add_action('plugins_loaded', [$this, 'init']);
    }
	
	//Init
    public function init() {

		if (!did_action('elementor/loaded')) {
			add_action('admin_notices', 'acfe_addon_load_fail');
			return;
        }        
		
        add_action( 'elementor/init', [ $this, 'add_elementor_category' ] );
        add_action('elementor/frontend/after_enqueue_styles', [$this, 'includes']);
		add_action('elementor/widgets/widgets_registered', [$this, 'register_widgets']);
		
		add_action('admin_init', [$this, 'display_notice']);		
		add_action( 'admin_enqueue_scripts', [$this, 'admin_includes'] );
		load_plugin_textdomain('acfe', false, dirname(plugin_basename(__FILE__)) . '/languages' );
	}
	
	// Admin dismiss notice
	public function display_notice() {

		if(isset($_GET['acfe_dismiss']) && $_GET['acfe_dismiss'] == 1) {
	        add_option('acfe_dismiss' , true);
	    }
		
		$dismiss = get_option('acfe_dismiss');
		
		if(!get_option('acfe-top-notice')){
			add_option('acfe-top-notice',strtotime(current_time('mysql')));
		}
		
		if(get_option('acfe-top-notice') && get_option('acfe-top-notice') != 0) {
			if( get_option('acfe-top-notice') < strtotime('-10 days')) {			
				add_action('admin_notices', 			array($this,'acfe_top_admin_notice'));
				add_action('wp_ajax_acfe_top_notice',	array($this,'acfe_top_notice_dismiss'));
			}
		}
	}
	
	// Admin dismiss notice
	public function acfe_top_notice_dismiss(){
		update_option('acfe-top-notice',strtotime(current_time('mysql')));
		exit();
	}
	
	// Admin notice
	public function acfe_top_admin_notice(){
		?>
			<style>.acfe-notice.notice-success{border-left-color:#d84242;background:rgba(216, 66, 66, 0.15);}</style>
			<div class="acfe-notice notice notice-success is-dismissible" style="text-align:center;padding:10px 0;display:flex;align-items:center;justify-content:center;flex-direction:column;">
				<p style="width:100%;"><?php echo esc_html__('Enjoying our ').'<strong>'.esc_html__('Advanced Chart for Elementor?').'</strong>'.esc_html__(' We hope you liked it! If you feel this plugin helped you, You can give us a 5 star rating!').'<br>'. esc_html__('It will motivate us to serve you more !','acfe'); ?> </p>				
				<a href="https://kapasias.com/" class="button button-secondary" target="_blank" style="background:#d84242;color:#fff;border-radius:50px;outline:none;border:1px solid #d84242;">
				<?php echo esc_html__('VISIT NOW','acfe'); ?></a>				
			</div>
		<?php
	}
	
	// Elementor category
	public function add_elementor_category() {
			
		$elementor = \Elementor\Plugin::$instance;
		$elementor->elements_manager->add_category('kap-asia', 
			[
				'title' => esc_html__( 'KAP ASIAs', 'acfe' ),				
			],
			1
		);	
	}
	
	// Register widget
    public function register_widgets() {
        require_once(ACFE_PATH . 'include/acfe-advanced-chart.php');
    }
	
	// Js and Css
    public function includes() {		
		wp_enqueue_script( 'acfe-third-party-editor-js', ACFE_URL . 'assets/js/extra/chart.js',array('jquery'), ACFE_VERSION,true);
		wp_enqueue_script( 'acfe-editor-js', ACFE_URL . 'assets/js/main/acfe_script.min.js',array('jquery'), ACFE_VERSION,true);
		
        if (isset($_GET['elementor-preview']) || (isset($_REQUEST['action']) && $_REQUEST['action'] == 'elementor')) {
           wp_enqueue_style('acfe-la-css');
        }
    }
	
	// Admin side notice
	public function admin_includes() {
		wp_enqueue_script( 'acfe-editor-js-note', ACFE_URL . 'assets/js/admin/acfe_script_note.js',array('jquery'), ACFE_VERSION,true);
    }
    
}

Ka_Advanced_Chart_Addon::get_instance();

?>