<?php 
namespace Elementor;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Utils;
use Elementor\Scheme_Color;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Scheme_Typography;
use Elementor\Group_Control_Image_Size;

// Prevent direct access to files
if (!defined('ABSPATH')) {
    exit;
}

class Advanced_Chart_Addon_Kap_Asias extends Widget_Base {
	
	public function get_name() {
		return 'acfe-advanced-chart';
	}

    public function get_title() {
        return esc_html__('Advanced Chart', 'acfe');
    }

    public function get_icon() {
        return 'fa fa-line-chart';
    }

    public function get_categories() {
        return array('kap-asia');
    }
	
	public function get_keywords() {
		return ['advanced chart','chart','line','bar','vertical bar','horizontal bar','radar','pie','doughnut','polararea','bubble','scatter','datachart'];
	}
	
    protected function _register_controls() {
		
		$this->start_controls_section(
			'advanced_chart_content',
			[
				'label' => esc_html__( 'Advanced Chart', 'acfe' ),
				'tab' => Controls_Manager::TAB_CONTENT,
			]
		);
		$this->add_control(
			'select_chart',
			[
				'label' => esc_html__( 'Chart Type', 'acfe' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'line',
				'options' => [
					'line'  => esc_html__( 'Line', 'acfe' ),
					'bar' => esc_html__( 'Vertical Bar', 'acfe' ),
					'horizontalBar' => esc_html__( 'Horizontal Bar', 'acfe' ),
					'radar' => esc_html__( 'Radar', 'acfe' ),
					'pie' => esc_html__( 'Pie', 'acfe' ),
					'doughnut' => esc_html__( 'Doughnut', 'acfe' ),
					'polarArea' => esc_html__( 'Polar Area', 'acfe' ),
					'bubble' => esc_html__( 'Bubble', 'acfe' ),
					'scatter' => esc_html__( 'Scatter', 'acfe' ),
				], 
			]
		);
		$this->add_control(
			'main_label',
			[
				'label' => esc_html__( 'label Values', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( 'Jan , Feb , Mar', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ),  
				'dynamic' => ['active' => true,], 
				'condition' => [
					'select_chart!' => 'scatter',
				]
			]
		);
		$this->end_controls_section();
		/*line,bar,radar dataset start*/
		$this->start_controls_section(
            'chart_dataset',
            [
                'label' => esc_html__('Dataset', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT,
				'condition' => [
					'select_chart' => [ 'line', 'bar','horizontalBar','radar']
				]
            ]
        ); 
		$repeater = new \Elementor\Repeater();  
		$repeater->add_control(
			'loop_label',
			[
				'label' => esc_html__( 'Label', 'acfe' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Label', 'acfe' ),
				'placeholder' => esc_html__( 'Enter label', 'acfe' ),  
				'dynamic' => ['active'   => true,],
				 				
			]
		);
		$repeater->add_control(
			'loop_data',
			[
				'label' => esc_html__( 'Data', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '0 , 10 , 35', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ), 
				'dynamic' => ['active'   => true,],
				'separator' => 'before',
			]
		);
		$repeater->add_control(
			'individual_dot_bg',
			[
				'label' => esc_html__( 'Individual Dot Background', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'return_value' => 'yes',
				'default' => 'no',
				'separator' => 'before',
			]
		);
		$repeater->add_control(
			'dot_bg',
			[
				'label' => esc_html__( 'Dot Background', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => 'rgb(0 0 0 / 0.5)',
				'condition' => [
					'individual_dot_bg!' => 'yes',
				],
			]
		);
		$repeater->add_control(
			'individual_dot_bg_pass',
			[
				'label' => esc_html__( 'Dot Background', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '#bd3b1b80,#d6a80080,#b9d87080', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by , ', 'acfe' ),  
				'dynamic' => ['active'   => true,],
				'condition'    => [
					'individual_dot_bg' => 'yes',
				],
			]
		);
		$repeater->add_control(
			'individual_dot_border',
			[
				'label' => esc_html__( 'Individual Border', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'return_value' => 'yes',
				'default' => 'no',
				'separator' => 'before',
			]
		);
		$repeater->add_control(
			'dot_border',
			[
				'label' => esc_html__( 'Dot Border', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => 'rgb(0 0 0 / 0.5)',
				'condition' => [
					'individual_dot_border!' => 'yes',
				],
			]
		);
		$repeater->add_control(
			'individual_dot_border_pass',
			[
				'label' => esc_html__( 'Dot Border', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '#bd3b1b80,#d6a80080,#b9d87080', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ),  
				'dynamic' => ['active'   => true,],
				'condition'    => [
					'individual_dot_border' => 'yes',
				],
			]
		);
		$repeater->add_control(
			'fill_opt',
			[
				'label' => esc_html__( 'Fill', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'yes',
				'description' => esc_html__( 'Please Note : Fill options works in Line and Radar chart only', 'acfe' ),
				'separator' => 'before',
			]
		);
		$repeater->add_control(
			'line_styling_opt',
			[
				'label' => esc_html__( 'Border Dash', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'no',
				'description' => esc_html__( 'Please Note : Fill options works in Line and Radar chart only', 'acfe' ),
				'separator' => 'before',
			]
		);
		$this->add_control(
			'chart_content',
			[
				'label' => esc_html__( 'Chart Data Boxes', 'acfe' ),
				'type' => Controls_Manager::REPEATER,
				'fields' => $repeater->get_controls(),
				'default' => [
                    [  
						'loop_label' => esc_html__( 'Jan', 'acfe' ),
						'loop_data' => esc_html__( '0 , 10 , 35 ', 'acfe' ),						
						'dot_bg'  => '#bd3b1b80',
						'dot_border' => '#bd3b1b80',
                    ],
					[ 
						'loop_label' => esc_html__( 'Feb', 'acfe' ),
						'loop_data' => esc_html__( '5 , 12 , 46', 'acfe' ),
						'dot_bg'  => '#d6a80080',
						'dot_border' => '#d6a80080',
                    ],
					[ 
						'loop_label' => esc_html__( 'Mar', 'acfe' ),
						'loop_data' => esc_html__( '15 , 20 , 60', 'acfe' ),
						'dot_bg'  => '#b9d87080',
						'dot_border' => '#b9d87080',
                    ],					
                ],
				'title_field' => '{{{ loop_label }}}',
			]
		);
		$this->end_controls_section();
		/*line,bar,radar dataset end*/
		
		/*pie dataset start*/
		$this->start_controls_section(
            'chart_dataset_seprate',
            [
                'label' => esc_html__('Dataset', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT,
				'condition' => [
					'select_chart' => ['pie','polarArea'],
				]
            ]
        );		
		$this->add_control(
			'seprate_data',
			[
				'label' => esc_html__( 'Data', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '5 , 10 , 15', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ), 
				'dynamic' => ['active'   => true,],
				'separator' => 'before',
			]
		);
		$this->add_control(
			'seprate_bg_colors',
			[
				'label' => esc_html__( 'Background Colors', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '#bd3b1b80,#d6a80080,#b9d87080', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ), 
				'dynamic' => ['active'   => true,],
				'separator' => 'before',
			]
		);
		$this->add_control(
			'seprate_border_colors',
			[
				'label' => esc_html__( 'Border Colors', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '#bd3b1b80,#d6a80080,#b9d87080', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ), 
				'dynamic' => ['active'   => true,],
				'separator' => 'before',
				'condition' => [
					'select_chart' => 'pie',
				]
			]
		);
		$this->add_control(
			'seprate_border_colors_polar',
			[
				'label' => esc_html__( 'Border Colors', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '#bd3b1b80,#d6a80080,#b9d87080', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ), 
				'dynamic' => ['active'   => true,],
				'separator' => 'before',
				'condition' => [
					'select_chart' => 'polarArea',
				]
			]
		);
		$this->end_controls_section();
		/*pie dataset end*/
		
		/*doughnut dataset start*/
		$this->start_controls_section(
            'doughnut_chart_dataset',
            [
                'label' => esc_html__('Dataset', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT,
				'condition' => [
					'select_chart' => 'doughnut',
				]
            ]
        );
		$repeater2 = new \Elementor\Repeater(); 
		$repeater2->add_control(
			'doughnut_loop_label',
			[
				'label' => esc_html__( 'Label', 'acfe' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Label', 'acfe' ),
				'placeholder' => esc_html__( 'Enter label', 'acfe' ),  
				'dynamic' => ['active'   => true,],
				 				
			]
		);
		$repeater2->add_control(
			'doughnut_loop_data',
			[
				'label' => esc_html__( 'Data', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '5 , 10 , 15', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ), 
				'dynamic' => ['active'   => true,],
				'separator' => 'before',
			]
		);
		$repeater2->add_control(
			'doughnut_loop_background',
			[
				'label' => esc_html__( 'Background Colors', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '#b6c61a80,#b9d87080,#d6a80080', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ), 
				'dynamic' => ['active'   => true,],
				'separator' => 'before',
			]
		);
		$repeater2->add_control(
			'doughnut_loop_border',
			[
				'label' => esc_html__( 'Border Colors', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '#00634480,#00634480,#00634480', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ), 
				'dynamic' => ['active'   => true,],
				'separator' => 'before',
			]
		);
		$this->add_control(
			'doughnut_chart_datasets',
			[
				'label' => esc_html__( 'Chart Data Boxes', 'acfe' ),
				'type' => Controls_Manager::REPEATER,
				'fields' => $repeater2->get_controls(),
				'default' => [
                    [  
						'doughnut_loop_label' => esc_html__( 'Jan', 'acfe' ),
						'doughnut_loop_data' => esc_html__( '5 , 10 , 15 ', 'acfe' ),						
						'doughnut_loop_background'  => '#b6c61a80,#b9d87080,#d6a80080',
						'doughnut_loop_border' => '#00634480,#00634480,#00634480',
                    ],
					[ 
						'doughnut_loop_label' => esc_html__( 'Feb', 'acfe' ),
						'doughnut_loop_data' => esc_html__( '7 , 12 , 24', 'acfe' ),
						'doughnut_loop_background'  => '#bd3b1b80,#d6a80080,#b9d87080',
						'doughnut_loop_border' => '#40e0d0,#40e0d0,#40e0d0',
                    ],
					[ 
						'doughnut_loop_label' => esc_html__( 'Mar', 'acfe' ),
						'doughnut_loop_data' => esc_html__( '15 , 21 , 45', 'acfe' ),
						'doughnut_loop_background'  => '#71d1dc99,#b9d87080,#b6c61a80',
						'doughnut_loop_border' => '#00634480,#00634480,#00634480',
                    ],					
                ],
				'title_field' => '{{{ doughnut_loop_label }}}',
			]
		);
		$this->end_controls_section(); 
		/*doughnut dataset end*/
		
		/*bubble dataset start*/
		$this->start_controls_section(
            'chart_dataset_bubble',
            [
                'label' => esc_html__('Datasets', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT,
				'condition' => [
					'select_chart' => 'bubble'
				]
            ]
        );
		$repeater_bubble = new \Elementor\Repeater();  
		$repeater_bubble->add_control(
			'loop_label',
			[
				'label' => esc_html__( 'Label', 'acfe' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Label', 'acfe' ),
				'placeholder' => esc_html__( 'Enter label', 'acfe' ),  
				'dynamic' => ['active'   => true,],
				 				
			]
		);
		$repeater_bubble->add_control(
			'bubble_values',
			[
				'label' => esc_html__( 'Data', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '[5,10,15][7,18,9][8,12,20]', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ), 
				'dynamic' => ['active'   => true,],
				'separator' => 'before',
			]
		);
		$repeater_bubble->add_control(
			'individual_dot_bg',
			[
				'label' => esc_html__( 'Individual Background Colors', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'return_value' => 'yes',
				'default' => 'no',
				'separator' => 'before',
			]
		);
		$repeater_bubble->add_control(
			'dot_bg',
			[
				'label' => esc_html__( 'Background', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => 'rgb(0 0 0 / 0.5)',
				'condition' => [
					'individual_dot_bg!' => 'yes',
				],
			]
		);
		$repeater_bubble->add_control(
			'individual_dot_bg_pass',
			[
				'label' => esc_html__( 'Background', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '#bd3b1b80,#d6a80080,#b9d87080', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ),  
				'dynamic' => ['active'   => true,],
				'condition'    => [
					'individual_dot_bg' => 'yes',
				],
			]
		);
		$repeater_bubble->add_control(
			'individual_dot_border',
			[
				'label' => esc_html__( 'Individual Border Colors', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'return_value' => 'yes',
				'default' => 'no',
				'separator' => 'before',
			]
		);
		$repeater_bubble->add_control(
			'dot_border',
			[
				'label' => esc_html__( 'Dot Border', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => 'rgb(0 0 0 / 0.5)',
				'condition' => [
					'individual_dot_border!' => 'yes',
				],
			]
		);
		$repeater_bubble->add_control(
			'individual_dot_border_pass',
			[
				'label' => esc_html__( 'Dot Background Colors', 'acfe' ),
				'type' => Controls_Manager::TEXTAREA,
				'default' => esc_html__( '#bd3b1b80,#d6a80080,#b9d87080', 'acfe' ),
				'placeholder' => esc_html__( 'Seprate by ,(Comma) ', 'acfe' ),  
				'dynamic' => ['active'   => true,],
				'condition'    => [
					'individual_dot_border' => 'yes',
				],
			]
		);
		$this->add_control(
			'bubble_content',
			[
				'label' => esc_html__( 'Chart Data Boxes', 'acfe' ),
				'type' => Controls_Manager::REPEATER,
				'fields' => $repeater_bubble->get_controls(),
				'default' => [
                    [  
						'loop_label' => esc_html__( 'Jan', 'acfe' ),
						'bubble_values' => esc_html__( '[5,10,15][7,18,9][8,14,20]', 'acfe' ),						
						'dot_bg'  => '#bd3b1b80',
						'dot_border' => '#bd3b1b80',
                    ],
					[ 
						'loop_label' => esc_html__( 'Feb', 'acfe' ),
						'bubble_values' => esc_html__( '[7,8,17][5,20,7][10,12,14]', 'acfe' ),
						'dot_bg'  => '#d6a80080',
						'dot_border' => '#d6a80080',
                    ],
					[ 
						'loop_label' => esc_html__( 'Mar', 'acfe' ),
						'bubble_values' => esc_html__( '[6,9,16][6,19,8][9,14,16]', 'acfe' ),
						'dot_bg'  => '#b9d87080',
						'dot_border' => '#b9d87080',
                    ],					
                ],
				'title_field' => '{{{ loop_label }}}',
			]
		);
		$this->end_controls_section(); 
		/*bubble dataset end*/
		
		/*bar options start*/
		$this->start_controls_section(
            'chart_bar_options',
            [
                'label' => esc_html__('Bar Options', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT, 
				'condition' => [
					'select_chart' => ['bar','horizontalBar'],
				],
            ]
        );
		$this->add_responsive_control(
            'maxbarthickness',
            [
                'type' => Controls_Manager::SLIDER,
				'label' => esc_html__('Bar Size', 'acfe'),
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 50,
						'step' => 1,
					],
				],				
            ]
        );
		$this->add_responsive_control(
            'barthickness',
            [
                'type' => Controls_Manager::SLIDER,
				'label' => esc_html__('Bar Space', 'acfe'),
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 50,
						'step' => 1,
					],
				],
            ]
        );
		$this->end_controls_section(); 
		/*bar options end*/
		
		/*grid lines options start*/
		$this->start_controls_section(
            'chart_grid_lines_options',
            [
                'label' => esc_html__('Grid Lines Options', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT, 
				'condition'    => [
					'select_chart!' => ['pie','doughnut','scatter'],
				],
            ]
        );
		$this->add_control(
			'show_grid_lines',
			[
				'label' => esc_html__( 'Grid Lines', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'yes',				
			]
		);
		$this->start_controls_tabs( 'tabs_axes_style' );
		$this->start_controls_tab(
			'tab_axes_x',
			[
				'label' => esc_html__( 'X Axes', 'acfe' ),
				'condition' => [
					'show_grid_lines' => 'yes',
				]
			]
		);
		$this->add_control(
			'grid_color',
			[
				'label' => esc_html__( 'Color', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => 'rgb(0 0 0 / 0.5)',
				'condition' => [
					'show_grid_lines' => 'yes',
				]
			]
		);
		$this->add_control(
			'zero_linecolor_x',
			[
				'label' => esc_html__( 'Zero Line Color', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => 'rgba(0, 0, 0, 0.25)',
				'condition' => [
					'show_grid_lines' => 'yes',
				]
			]
		);
		$this->add_control(
			'draw_border_x',
			[
				'label' => esc_html__( 'Draw Border', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'no',
				'condition' => [
					'show_grid_lines' => 'yes',
				]
			]
		);
		$this->add_control(
			'draw_on_chart_area_x',
			[
				'label' => esc_html__( 'Draw Border on Chart Area', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'yes',
				'condition' => [
					'show_grid_lines' => 'yes',
				]
			]
		);
		$this->end_controls_tab();
		$this->start_controls_tab(
			'tab_axes_y',
			[
				'label' => esc_html__( 'Y Axes', 'acfe' ),
				'condition' => [
					'show_grid_lines' => 'yes',
				]
			]
		);
		$this->add_control(
			'grid_color_xAxes',
			[
				'label' => esc_html__( 'Color', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => 'rgb(0 0 0 / 0.5)',
				'condition' => [
					'show_grid_lines' => 'yes',
				]
			]
		);
		$this->add_control(
			'zero_linecolor_y',
			[
				'label' => esc_html__( 'Zero Line Color', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => 'rgba(0, 0, 0, 0.25)',
				'condition' => [
					'show_grid_lines' => 'yes',
				]
			]
		);
		$this->add_control(
			'draw_border_y',
			[
				'label' => esc_html__( 'Draw Border', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'no',
				'condition' => [
					'show_grid_lines' => 'yes',
				]
			]
		);
		$this->add_control(
			'draw_on_chart_area_y',
			[
				'label' => esc_html__( 'Draw Border on Chart Area', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'yes',
				'condition' => [
					'show_grid_lines' => 'yes',
				]
			]
		);
		$this->end_controls_tab();
		$this->end_controls_tabs();
		$this->end_controls_section();
		/*grid lines options end*/
		
		/*labels options start*/
		$this->start_controls_section(
            'chart_labels_options',
            [
                'label' => esc_html__('Labels Options', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT, 
				'condition' => [
					'select_chart!' => ['pie','doughnut','scatter'],
				],
            ]
        );
		$this->add_control(
			'show_labels',
			[
				'label' => esc_html__( 'Labels', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'yes',
			]
		);
		$this->add_control(
			'show_labels_color',
			[
				'label' => esc_html__( 'Color', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => '#666',
				'condition' => [
					'show_labels' => 'yes',
				]
			]
		);
		$this->add_responsive_control(
            'show_labels_size',
            [
                'type' => Controls_Manager::SLIDER,
				'label' => esc_html__('Size', 'acfe'),
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 50,
						'step' => 1,
					],
				],
				'default' => [					
					'size' => 12,
				],
				'condition' => [
					'show_labels' => 'yes',
				],
            ]
        );
		$this->end_controls_section();
		/*labels options end*/
		
		/*legend options start*/
		$this->start_controls_section(
            'chart_legend_options',
            [
                'label' => esc_html__('Legend Options', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT,
				'condition'    => [
					'select_chart!' => 'scatter',
				],
            ]
        );
		$this->add_control(
			'show_legend',
			[
				'label' => esc_html__( 'Legend', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'yes',
			]
		);	
		$this->add_responsive_control(
            'show_legend_size',
            [
                'type' => Controls_Manager::SLIDER,
				'label' => esc_html__('Size', 'acfe'),
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 50,
						'step' => 1,
					],
				],
				'default' => [					
					'size' => 12,
				],
				'condition' => [
					'show_legend' => 'yes',
				],
            ]
        );
		$this->add_control(
			'show_legend_color',
			[
				'label' => esc_html__( 'Color', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => '#666',
				'condition' => [
					'show_legend' => 'yes',
				],
			]
		);
		$this->add_control(
			'show_legend_position',
			[
				'label' => esc_html__( 'Position', 'acfe' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'acfe' ),
						'icon' => 'fa fa-arrow-left',
					],
					'top' => [
						'title' => esc_html__( 'Top', 'acfe' ),
						'icon' => 'fa fa-arrow-up',
					],
					'bottom' => [
						'title' => esc_html__( 'Bottom', 'acfe' ),
						'icon' => 'fa fa-arrow-down',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'acfe' ),
						'icon' => 'fa fa-arrow-right',
					],
				],
				'default' => 'top',
				'condition' => [
					'show_legend' => 'yes',
				],
			]
		);
		$this->add_control(
			'show_legend_align',
			[
				'label' => esc_html__( 'Alignment', 'acfe' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'start' => [
						'title' => esc_html__( 'Start', 'acfe' ),
						'icon' => 'fa fa-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'acfe' ),
						'icon' => 'fa fa-align-center',
					],
					'end' => [
						'title' => esc_html__( 'End', 'acfe' ),
						'icon' => 'fa fa-align-right',
					],
				],
				'default' => 'center',
				'condition' => [
					'show_legend' => 'yes',
				],
			]
		);
		$this->end_controls_section();
		/*legend options end*/
		
		/*tooltip options start*/
		$this->start_controls_section(
            'chart_tooltip_options',
            [
                'label' => esc_html__('Tooltip Options', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT,
				'condition'    => [
					'select_chart!' => 'scatter',
				],
            ]
        );
		$this->add_control(
			'show_tooltip',
			[
				'label' => esc_html__( 'Tooltip', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'yes',
			]
		);
		$this->add_control(
			'tooltip_event',
			[
				'label' => esc_html__( 'Event', 'acfe' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'hover',
				'options' => [
					'hover'  => esc_html__( 'Hover', 'acfe' ),
					'click' => esc_html__( 'Click', 'acfe' ),
				],
				'condition' => [
					'show_tooltip' => 'yes',
				],
			]
		);
		$this->add_responsive_control(
            'tooltip_font_size',
            [
                'type' => Controls_Manager::SLIDER,
				'label' => esc_html__('Font Size', 'acfe'),
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 50,
						'step' => 1,
					],
				],
				'default' => [					
					'size' => 12,
				],
				'condition' => [
					'show_tooltip' => 'yes',
				],
            ]
        );
		$this->add_control(
			'tooltip_color',
			[
				'label' => esc_html__( 'Title Color', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => '#fff',
				'condition' => [
					'show_tooltip' => 'yes',
				],
			]
		);
		$this->add_control(
			'tooltip_body_font_color',
			[
				'label' => esc_html__( 'Body Font Color', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => '#fff',
				'condition' => [
					'show_tooltip' => 'yes',
				],
			]
		);
		$this->add_control(
			'tooltip_bg',
			[
				'label' => esc_html__( 'Tooltip Background', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'condition' => [
					'show_tooltip' => 'yes',
				],
			]
		);
		$this->end_controls_section();
		/*tooltip options end*/
		
		/*animation options start*/
		$this->start_controls_section(
            'chart_animation_options',
            [
                'label' => esc_html__('Animation Options', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT,
				'condition'    => [
					'select_chart!' => 'scatter',
				],
            ]
        );
		$this->add_control(
			'custom_animation',
			[
				'label' => esc_html__( 'Animation', 'acfe' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'easeOutQuart',
				'options' => [
					'linear'  => esc_html__( 'linear', 'acfe' ),
					'easeInQuad' => esc_html__( 'easeInQuad', 'acfe' ),
					'easeOutQuad' => esc_html__( 'easeOutQuad', 'acfe' ),
					'easeInOutQuad' => esc_html__( 'easeInOutQuad', 'acfe' ),
					'easeInCubic' => esc_html__( 'easeInCubic', 'acfe' ),
					'easeOutCubic' => esc_html__( 'easeOutCubic', 'acfe' ),
					'easeInOutCubic' => esc_html__( 'easeInOutCubic', 'acfe' ),
					'easeInQuart' => esc_html__( 'easeInQuart', 'acfe' ),
					'easeOutQuart' => esc_html__( 'easeOutQuart', 'acfe' ),
					'easeInOutQuart' => esc_html__( 'easeInOutQuart', 'acfe' ),
					'easeInQuint'  => esc_html__( 'easeInQuint', 'acfe' ),
					'easeOutQuint' => esc_html__( 'easeOutQuint', 'acfe' ),
					'easeInOutQuint' => esc_html__( 'easeInOutQuint', 'acfe' ),
					'easeInSine' => esc_html__( 'easeInSine', 'acfe' ),
					'easeOutSine' => esc_html__( 'easeOutSine', 'acfe' ),
					'easeInOutSine' => esc_html__( 'easeInOutSine', 'acfe' ),
					'easeInExpo' => esc_html__( 'easeInExpo', 'acfe' ),
					'easeOutExpo' => esc_html__( 'easeOutExpo', 'acfe' ),
					'easeInOutExpo' => esc_html__( 'easeInOutExpo', 'acfe' ),
					'easeInCirc' => esc_html__( 'easeInCirc', 'acfe' ),
					'easeOutCirc'  => esc_html__( 'easeOutCirc', 'acfe' ),
					'easeInOutCirc' => esc_html__( 'easeInOutCirc', 'acfe' ),
					'easeInElastic' => esc_html__( 'easeInElastic', 'acfe' ),
					'easeOutElastic' => esc_html__( 'easeOutElastic', 'acfe' ),
					'easeInOutElastic' => esc_html__( 'easeInOutElastic', 'acfe' ),
					'easeInBack' => esc_html__( 'easeInBack', 'acfe' ),
					'easeOutBack' => esc_html__( 'easeOutBack', 'acfe' ),
					'easeInOutBack' => esc_html__( 'easeInOutBack', 'acfe' ),
					'easeInBounce' => esc_html__( 'easeInBounce', 'acfe' ),
					'easeOutBounce' => esc_html__( 'easeOutBounce', 'acfe' ),
					'easeInOutBounce'  => esc_html__( 'easeInOutBounce', 'acfe' ),
				],
			]
		);
		$this->add_responsive_control(
            'custom_animation_duration',
            [
                'type' => Controls_Manager::SLIDER,
				'label' => esc_html__('Duration', 'acfe'),
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 10000,
						'step' => 100,
					],
				],
				'default' => [					
					'size' => 1000,
				],
            ]
        );
		$this->end_controls_section();
		/*animation options end*/
		
		/*advanced options start*/
		$this->start_controls_section(
            'chart_advanced_options',
            [
                'label' => esc_html__('Advanced Options', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT, 
				'condition'    => [
					'select_chart!' => 'scatter',
				],
            ]
        );		
		$this->add_control(
			'tension',
			[
				'label' => esc_html__( 'Smooth', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'yes',
				'separator' => 'before',
				'condition' => [
					'select_chart' => 'line',
				]
			]
		);
		$this->add_control(
			'customize_point_styles',
			[
				'label' => esc_html__( 'Customize Point Styles', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'no',
				'separator' => 'before',
				'condition' => [
					'select_chart' => ['line','radar','bubble'],
				]
			]
		);
		$this->add_control(
			'point_styles_select',
			[
				'label' => esc_html__( 'Point Styles', 'acfe' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'circle',
				'options' => [
					'circle'  => esc_html__( 'Circle', 'acfe' ),
					'cross' => esc_html__( 'Cross', 'acfe' ),
					'crossRot' => esc_html__( 'CrossRot', 'acfe' ),
					'dash' => esc_html__( 'Dash', 'acfe' ),
					'line' => esc_html__( 'Line', 'acfe' ),
					'rect' => esc_html__( 'Rect', 'acfe' ),
					'rectRounded' => esc_html__( 'RectRounded', 'acfe' ),
					'rectRot' => esc_html__( 'RectRot', 'acfe' ),
					'star' => esc_html__( 'Star', 'acfe' ),
					'triangle' => esc_html__( 'Triangle', 'acfe' ),
				], 
				'condition' => [
					'select_chart' => ['line','radar','bubble'],
					'customize_point_styles' => 'yes',
				]
			]
		);
		$this->add_control(
			'point_bg',
			[
				'label' => esc_html__( 'Point Background', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => '#b6c61a80',
				'condition' => [
					'select_chart' => ['line','radar'],
					'customize_point_styles' => 'yes',
				],
			]
		);
		$this->add_responsive_control(
            'point_n_size',
            [
                'type' => Controls_Manager::SLIDER,
				'label' => esc_html__('Point Normal Size', 'acfe'),
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 50,
						'step' => 1,
					],
				],
				'default' => [					
					'size' => 5,
				],
				'condition' => [
					'select_chart' => ['line','radar'],
					'customize_point_styles' => 'yes',
				],
            ]
        );
		$this->add_responsive_control(
            'point_h_size',
            [
                'type' => Controls_Manager::SLIDER,
				'label' => esc_html__('Point Hover Size', 'acfe'),
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 50,
						'step' => 1,
					],
				],
				'default' => [					
					'size' => 10,
				],
				'condition' => [
					'select_chart' => ['line','radar'],
					'customize_point_styles' => 'yes',
				],
            ]
        );
		$this->add_control(
			'point_border_color',
			[
				'label' => esc_html__( 'Point Border', 'acfe' ),
				'type' => Controls_Manager::COLOR,
				'default'     => '#00634480',
				'condition' => [
					'select_chart' => ['line','radar'],
					'customize_point_styles' => 'yes',
				],
			]
		);
		$this->add_responsive_control(
            'point_border_width',
            [
                'type' => Controls_Manager::SLIDER,
				'label' => esc_html__('Point Border Width', 'acfe'),
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 50,
						'step' => 1,
					],
				],
				'default' => [					
					'size' => 1,
				],
				'condition' => [
					'select_chart' => ['line','radar'],
					'customize_point_styles' => 'yes',
				],
            ]
        );			
		$this->add_control(
			'aspect_ratio',
			[
				'label' => esc_html__( 'Aspect Ratio', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'no',
				'separator' => 'before',
			]
		);
		$this->add_control(
			'maintain_aspect_ratio',
			[
				'label' => esc_html__( 'Maintain Aspect Ratio', 'acfe' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Enable', 'acfe' ),
				'label_off' => esc_html__( 'Disable', 'acfe' ),
				'default' => 'yes',
				'separator' => 'before',
			]
		);		
		$this->end_controls_section();
		/*advanced options end*/

		/*comming soon start*/
		$this->start_controls_section(
            'chart_comming_soon_options',
            [
                'label' => esc_html__('Advanced Options', 'acfe'),
                'tab' => Controls_Manager::TAB_CONTENT, 
				'condition'    => [
					'select_chart' => 'scatter',
				],
            ]
        );
		$this->add_control(
			'important_note',
			[
				
				'type' => \Elementor\Controls_Manager::RAW_HTML,
				'raw' => __( 'Comming Soon', 'acfe' ),
			]
		);
		$this->end_controls_section();
		/*comming soon  end*/
	}
	
	public function bubble_values( $bubble_values ) {
		$bubble_values = trim( $bubble_values );		
		$divide_value = preg_match_all( '#\[([^\]]+)\]#U', $bubble_values, $get_and_verify );		
		if ( !$divide_value ) {
			return [];
		}else {
			$bubble_value = $get_and_verify[1];
			$bubble = [];
			foreach ( $bubble_value as $item_value ) {
				$item_value = trim( $item_value, '][ ' );
				$item_value = explode( ',', $item_value );
				
				if (count($item_value) != 3){
					continue;
				}					
				
				$x_y_r = new \stdClass();
				$x_y_r->x = floatval( trim( $item_value[0] ) );
				$x_y_r->y = floatval( trim( $item_value[1] ) );
				$x_y_r->r = floatval( trim( $item_value[2] ) );
				$bubble[] = $x_y_r;
			}
			return $bubble;
		}
	}
	protected function render() { 	
		$settings = $this->get_settings_for_display(); 
		$acfe = $label_data = $get_data = '';
		$select_chart = $settings['select_chart'];
		
		$get_values=$datasets=$datasets1=$chart_datasets=[];
		
		if(!empty($select_chart) && $select_chart!='scatter'){
			if($select_chart=='pie' || $select_chart=='doughnut' || $select_chart=='polarArea'){
					if($select_chart=='pie' || $select_chart=='polarArea'){
						$seprate_data = array_map('floatval', explode(',', $settings['seprate_data']));
						if(!empty($seprate_data)){
							$datasets[] = [ 
								'data' => $seprate_data,
								'backgroundColor' => explode(',', $settings['seprate_bg_colors'])
							];
						}
						
						if($select_chart=='pie' && !empty($settings['seprate_border_colors'])){
							$datasets[] = [
								'borderColor' => explode(',', $settings['seprate_border_colors'])
							];
						}
						
						if($select_chart=='polarArea' && !empty($settings['seprate_border_colors_polar'])){
							$datasets[] = [
								'borderColor' => explode(',', $settings['seprate_border_colors_polar'])
							];
						}
					}else if($select_chart=='doughnut'){
						foreach($settings['doughnut_chart_datasets'] as $item1){
							
							$datasets2['data']  =  array_map('floatval', explode(',', $item1['doughnut_loop_data']));
							
							$datasets2['backgroundColor'] = ($item1['doughnut_loop_background']) ? explode(',', $item1['doughnut_loop_background']) : [];
							
							$datasets2['borderColor'] = ($item1['doughnut_loop_border']) ? explode(',', $item1['doughnut_loop_border']) : [];
							
							$datasets[] = $datasets2;
						}
					}			
					
				}else{
					$chart_datasets = ($select_chart=='bubble') ?  $settings['bubble_content'] : $settings['chart_content'];			
					foreach($chart_datasets as $item){
						$datasets1['label'] = $item['loop_label'];

						if (!empty($select_chart) && $select_chart=='bubble') {
							$datasets1['data'] = $this->bubble_values($item['bubble_values']);
						} else {
							$datasets1['data']  =  array_map('floatval', explode(',', $item['loop_data']));				
						}

						if((!empty($item['individual_dot_bg']) && $item['individual_dot_bg']=='yes') && !empty($item['individual_dot_bg_pass'])) {
							$datasets1['backgroundColor'] = explode(',', $item['individual_dot_bg_pass']);
						} else {
							$datasets1['backgroundColor'] = $item['dot_bg'];
						}

						if((!empty($item['individual_dot_border']) && $item['individual_dot_border']=='yes') && !empty($item['individual_dot_border_pass'])){
							$datasets1['borderColor'] = explode(',', $item['individual_dot_border_pass']);
						} else {
							$datasets1['borderColor'] = $item['dot_border'];
						}
						
						$datasets1['borderDash']=[];
						if(($select_chart=='line' || $select_chart=='radar') && (!empty($item['line_styling_opt']) && $item['line_styling_opt']=='yes')){
							$datasets1['borderDash'] = [5, 5];
						}
						
						if(!empty($item['fill_opt']) && $item['fill_opt']=='yes'){
							 $datasets1['fill'] =true;
						}else{
							 $datasets1['fill'] =false;
						}
						
						if($select_chart=='line' || $select_chart=='radar' || $select_chart=='bubble'){					
							if(!empty($settings['customize_point_styles']) && $settings['customize_point_styles']=='yes'){
								if(!empty($settings['point_styles_select'])){
									$datasets1['pointStyle'] =$settings['point_styles_select'];
								}
								if(!empty($settings['point_bg']) && $select_chart!='bubble'){
									$datasets1['pointBackgroundColor'] =$settings['point_bg'];
								}
								if(!empty($settings['point_n_size']['size']) && $select_chart!='bubble'){
									$datasets1['pointRadius'] =$settings['point_n_size']['size'];
								}
								if(!empty($settings['point_h_size']['size']) && $select_chart!='bubble'){
									$datasets1['pointHoverRadius'] =$settings['point_h_size']['size'];
								}
								if(!empty($settings['point_border_width']['size']) && $select_chart!='bubble'){
									$datasets1['borderWidth'] = $settings['point_border_width']['size'];
								}
								if(!empty($settings['point_border_color']) && $select_chart!='bubble'){
									$datasets1['pointBorderColor'] =$settings['point_border_color'];	
								}
								
								if($select_chart=='line' && (!empty($settings['tension']) && $settings['tension']=='yes')){
									 $datasets1['tension']= 0.4;
								}else{
									 $datasets1['tension']= 0;
								}
							}
						}
											
						if (!empty($select_chart) && ($select_chart=='bar' || $select_chart=='horizontalBar')){
							if(!empty($settings['barthickness']['size'])){
								/*space*/
								$datasets1['barThickness']= $settings['barthickness']['size'];
							}
							if(!empty($settings['maxbarthickness']['size'])){
								/*size*/
								$datasets1['maxBarThickness']= $settings['maxbarthickness']['size'];
							}
						}	

						$datasets[] = $datasets1;
					} 
				}
			
			if($select_chart=='pie'){
				$get_values['cutoutPercentage'] = 0;
			}else if($select_chart=='doughnut'){
				$get_values['cutoutPercentage'] = 50;
			}else{
				if(!empty($settings['show_grid_lines']) && $settings['show_grid_lines']=='yes'){
					$get_values['scales'] = [
						'yAxes' => [[
							'ticks' => [
								'display' => (!empty($settings['show_labels']) ) ? true : false,
								'fontColor' => $settings['show_labels_color'],
								'fontSize' => $settings['show_labels_size']['size'],
							],
							'gridLines' => [							
								'color'      => $settings['grid_color'],
								'zeroLineColor' => $settings['zero_linecolor_y'],
								'drawBorder' => ( !empty($settings['draw_border_y'] )) ? true : false,
								'drawOnChartArea' => (!empty( $settings['draw_on_chart_area_y'] )) ? true : false,
								
							]
						]],
						'xAxes' => [[
							'ticks' => [
								'display' => ( !empty($settings['show_labels'] )) ? true : false,
								'fontColor' => $settings['show_labels_color'],
								'fontSize' => $settings['show_labels_size']['size'],
							],
							'gridLines' => [							
								'color'      => $settings['grid_color_xAxes'],
								'zeroLineColor' => $settings['zero_linecolor_x'],
								'drawBorder' => ( !empty($settings['draw_border_x'] )) ? true : false,
								'drawOnChartArea' => (!empty( $settings['draw_on_chart_area_x'] )) ? true : false,							
							]
						]]
					];
				}else{
					$get_values['scales'] = [
						'yAxes' => [[
							'ticks' => [
								'display' => ( $settings['show_labels'] ) ? true : false,
								'fontColor' => $settings['show_labels_color'],
								'fontSize' => $settings['show_labels_size']['size'],
							],
							'gridLines' => [
								'display'    => false,
							]
						]],
						'xAxes' => [[
							'ticks' => [
								'display' => ( $settings['show_labels'] ) ? true : false,
								'fontColor' => $settings['show_labels_color'],
								'fontSize' => $settings['show_labels_size']['size'],
							],
							'gridLines' => [
								'display'    => false,
							]
						]]
					];
				}
			}
			
			if (!empty($settings['show_legend']) && $settings['show_legend']=='yes') {
				if (!empty($settings['show_legend_position'])){
					$get_values['legend'] = [ 
						'position' => $settings['show_legend_position'],
						'align' => $settings['show_legend_align'],
						'labels' => [
								'fontColor' => $settings['show_legend_color'],
								'fontSize' => $settings['show_legend_size']['size'],
						],
					];
				}
			}else{
				$get_values['legend'] = [ 'display' => false ];
			}
			
			if(!empty($settings['custom_animation']) && !empty($settings['custom_animation_duration']['size'])){
				$get_values['animation'] = [ 'duration' => $settings['custom_animation_duration']['size'] , 'easing' => $settings['custom_animation']];
			}
			
			if(!empty($settings['show_tooltip']) && $settings['show_tooltip']=='yes') {
				if (!empty($settings['tooltip_bg']) || !empty($settings['tooltip_color']) || !empty($settings['tooltip_body_font_color'])){
					$get_values['tooltips'] = [ 
						'backgroundColor' => $settings['tooltip_bg'],
						'titleFontColor' => $settings['tooltip_color'],
						'bodyFontColor' => $settings['tooltip_body_font_color'],
						'titleFontSize' => $settings['tooltip_font_size']['size'],
						'bodyFontSize' => $settings['tooltip_font_size']['size'],
					];
				}
				if(!empty($settings['tooltip_event']) && $settings['tooltip_event']=='click'){
					$get_values['events'] = ['click'];
				}
				
			}else{
				$get_values['tooltips'] = [ 'enabled' => false ];
			}
			
			if (!empty($settings['aspect_ratio']) && $settings['aspect_ratio']=='yes') {
				$get_values['aspectRatio'] = 1;
			}

			if (!empty($settings['maintain_aspect_ratio']) && $settings['maintain_aspect_ratio'] !='yes') {
				$get_values['maintainAspectRatio'] = false;
			}
			
			$this->add_render_attribute([
				'pass_all_value' => [
					'data-options' => [ wp_json_encode(array_filter([
							"type"        => $select_chart,
							"data"        => [
								"labels" => explode(",", $settings["main_label"]),
								"datasets" => $datasets,  
							],						
							"options" => $get_values,
						]))							
					]
				]
			]);
			
			$unique=uniqid('chart');
			$acfe .= '<div class="acfe-chart" data-id="'.esc_attr($unique).'" '.$this->get_render_attribute_string( 'pass_all_value' ).'>';
				$acfe .= '<canvas id="'.$unique.'"></canvas>';		
			$acfe .= '</div>';
			
			echo $acfe;
		}
	}	
}
Plugin::instance()->widgets_manager->register_widget_type( new Advanced_Chart_Addon_Kap_Asias() );